import 'package:camera/camera.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:geocoding/geocoding.dart';
import 'package:geolocator/geolocator.dart';
import 'package:image_gallery_saver/image_gallery_saver.dart';
import 'package:screenshot/screenshot.dart';

List<CameraDescription>? cameras;

class CameraScreen extends StatefulWidget {
  const CameraScreen(
      {this.child, this.buttonDistance = 20, this.width = 10, super.key});
  final double? width;
  final double? buttonDistance;
  final Widget? child;
  @override
  CameraScreenState createState() => CameraScreenState();
}

class CameraScreenState extends State<CameraScreen> {
  CameraController? controller;
  final ScreenshotController screenshotController = ScreenshotController();
  @override
  void initState() {
    super.initState();
    getAddressFromLatLng();
    controller = CameraController(cameras![0], ResolutionPreset.high);
    controller?.initialize().then((_) {
      if (!mounted) {
        return;
      }
      setState(() {});
    });
  }

  String address = 'Fetching address...';
  String lat = 'Fetching lat...';
  String long = 'Fetching long...';

  Future<void> getAddressFromLatLng() async {
    try {
      // Get the current position (latitude and longitude)
      Position position = await Geolocator.getCurrentPosition(
        desiredAccuracy: LocationAccuracy.high,
      );

      // Reverse geocode the coordinates to get the address
      List<Placemark> placemarks = await placemarkFromCoordinates(
        position.latitude,
        position.longitude,
      );

      // Get the first address from the list
      Placemark placemark = placemarks.first;

      setState(() {
        lat = '${position.latitude}';
        long = '${position.longitude}';
        address =
            '${placemark.street}, ${placemark.locality}, ${placemark.country}';
      });
    } catch (e) {
      setState(() {
        address = 'Could not fetch address.';
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    if (!controller!.value.isInitialized) {
      return Container();
    }
    return Column(
      children: [
        Screenshot(
          controller: screenshotController,
          child: Stack(
            children: [
              AspectRatio(aspectRatio: 1, child: CameraPreview(controller!)),
              // SizedBox(
              //     width: MediaQuery.of(context).size.width,
              //     child: CameraPreview(controller!)),
              widget.child ??
                  Positioned(
                    bottom: 0,
                    right: 0,
                    child: Container(
                        decoration: BoxDecoration(
                          color: Colors.black.withOpacity(0.5),
                          borderRadius: const BorderRadius.only(
                            topLeft: Radius.circular(10),
                            bottomLeft: Radius.circular(10),
                          ),
                        ),
                        // color: Colors.black.withOpacity(0.5),
                        margin: const EdgeInsets.all(6),
                        child: Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                address,
                                style: const TextStyle(
                                  color: Colors.white,
                                  fontSize: 14,
                                ),
                              ),
                              Text(
                                'Long $lat',
                                style: const TextStyle(
                                  color: Colors.white,
                                  fontSize: 14,
                                ),
                              ),
                              Text(
                                'Long $long',
                                style: const TextStyle(
                                  color: Colors.white,
                                  fontSize: 14,
                                ),
                              ),
                              Text(
                                '${DateTime.now()}',
                                style: const TextStyle(
                                  color: Colors.white,
                                  fontSize: 14,
                                ),
                              ),
                            ],
                          ),
                        )),
                  ),
            ],
          ),
        ),
        SizedBox(
          height: widget.buttonDistance,
        ),
        ElevatedButton(
          onPressed: () {
            showDialog(
                context: context,
                builder: (context) => AlertDialog(
                      title: const Text('ERROR'),
                      content: const Text(
                          'FAIL TO SAVE THE IMAGE INTO YOUR GALLERY'),
                      actions: [
                        TextButton(
                            onPressed: () {
                              Navigator.pop(context);
                            },
                            child: const Text('No')),
                        TextButton(
                            onPressed: () {
                              Navigator.pop(context);
                              _captureAndSaveScreenshot();
                            },
                            child: const Text('Yes')),
                      ],
                    ));
          },
          // onPressed: captureAndSaveScreenshot,
          child: const Text('Capture Photo'),
        ),
      ],
    );
  }

  Future<void> captureAndSaveScreenshot() async {
    try {
      // Capture the screenshot
      final imageBytes = await screenshotController.capture();

      // Save the screenshot to the gallery
      final result =
          await ImageGallerySaver.saveImage(Uint8List.fromList(imageBytes!));

      if (result != null && result['isSuccess']) {
      } else {}
    } catch (e) {
      return;
    }
  }

  Future<void> _captureAndSaveScreenshot() async {
    try {
      final imageFile = await screenshotController.capture();

      final result = await ImageGallerySaver.saveImage(imageFile!);
      if (result!['isSuccess']) {
        // Screenshot saved successfully
        // print('Screenshot saved to gallery.');
      } else {
        // Failed to save screenshot
        // print('Failed to save screenshot: ${result['error']}');
      }
    } catch (e) {
      // print('Error capturing and saving screenshot: $e');
    }
  }

  // void _takePicture() async {
  //   try {
  //     final XFile image = await controller!.takePicture();
  //     // Now you have captured the image, you can proceed to save it and get location data.
  //     // Implement location and saving logic here.
  //   } catch (e) {
  //     print(e);
  //   }
  // }

  @override
  void dispose() {
    controller!.dispose();
    super.dispose();
  }
}
